@@ -34,13 +34,6 @@ module Lockbox
     end
 
     module AttachedOne
-      if ActiveStorage::VERSION::MAJOR < 6
-        def attach(attachable)
-          attachable = encrypt_attachable(attachable) if encrypted?
-          super(attachable)
-        end
-      end
-
       def rotate_encryption!
         raise "Not encrypted" unless encrypted?
 
@@ -51,19 +44,6 @@ module Lockbox
     end
 
     module AttachedMany
-      if ActiveStorage::VERSION::MAJOR < 6
-        def attach(*attachables)
-          if encrypted?
-            attachables =
-              attachables.flatten.collect do |attachable|
-                encrypt_attachable(attachable)
-              end
-          end
-
-          super(attachables)
-        end
-      end
-
       def rotate_encryption!
         raise "Not encrypted" unless encrypted?
 
@@ -100,7 +80,7 @@ module Lockbox
 
     module Attachment
       def download
-        result = super
+        result = super(&nil)
 
         options = Utils.encrypted_options(record, name)
         # only trust the metadata when migrating
@@ -111,45 +91,70 @@ module Lockbox
           result = Utils.decrypt_result(record, name, options, result)
         end
 
-        result
+        if block_given?
+          io = StringIO.new(result)
+          chunk_size = 5.megabytes
+          while (chunk = io.read(chunk_size))
+            yield chunk
+          end
+        else
+          result
+        end
       end
 
-      def variant(*args)
+      def download_chunk(...)
+        # TODO raise error in 3.0
+        warn "[lockbox] WARNING: download_chunk not supported for encrypted files" if Utils.encrypted_options(record, name)
+        super
+      end
+
+      def variant(...)
         raise Lockbox::Error, "Variant not supported for encrypted files" if Utils.encrypted_options(record, name)
         super
       end
 
-      def preview(*args)
+      def preview(...)
         raise Lockbox::Error, "Preview not supported for encrypted files" if Utils.encrypted_options(record, name)
         super
       end
 
-      if ActiveStorage::VERSION::MAJOR >= 6
-        def open(**options)
-          blob.open(**options) do |file|
-            options = Utils.encrypted_options(record, name)
-            # only trust the metadata when migrating
-            # as earlier versions of Lockbox won't have it
-            # and it's not a good practice to trust modifiable data
-            encrypted = options && (!options[:migrating] || blob.metadata["encrypted"])
-            if encrypted
-              result = Utils.decrypt_result(record, name, options, file.read)
-              file.rewind
-              # truncate may not be available on all platforms
-              # according to the Ruby docs
-              # may need to create a new temp file instead
-              file.truncate(0)
-              file.write(result)
-              file.rewind
-            end
+      if ActiveStorage::VERSION::STRING.to_f == 7.1 && ActiveStorage.version >= "7.1.4"
+        def transform_variants_later
+          blob.instance_variable_set(:@lockbox_encrypted, true) if Utils.encrypted_options(record, name)
+          super
+        end
+      end
 
-            yield file
+      def open(**options)
+        blob.open(**options) do |file|
+          options = Utils.encrypted_options(record, name)
+          # only trust the metadata when migrating
+          # as earlier versions of Lockbox won't have it
+          # and it's not a good practice to trust modifiable data
+          encrypted = options && (!options[:migrating] || blob.metadata["encrypted"])
+          if encrypted
+            result = Utils.decrypt_result(record, name, options, file.read)
+            file.rewind
+            # truncate may not be available on all platforms
+            # according to the Ruby docs
+            # may need to create a new temp file instead
+            file.truncate(0)
+            file.write(result)
+            file.rewind
           end
+
+          yield file
         end
       end
     end
 
     module Blob
+      if ActiveStorage::VERSION::STRING.to_f == 7.1 && ActiveStorage.version >= "7.1.4"
+        def preview_image_needed_before_processing_variants?
+          !instance_variable_defined?(:@lockbox_encrypted) && super
+        end
+      end
+
       private
 
       def extract_content_type(io)
