@@ -60,7 +60,7 @@ module Lockbox
         class_eval do
           # Lockbox uses custom inspect
           # but this could be useful for other gems
-          if activerecord && ActiveRecord::VERSION::MAJOR >= 6
+          if activerecord
             # only add virtual attribute
             # need to use regexp since strings do partial matching
             # also, need to use += instead of <<
@@ -114,12 +114,6 @@ module Lockbox
                   k = lockbox_encrypted_attributes[k]
                 elsif values.key?(k)
                   v = respond_to?(:attribute_for_inspect) ? attribute_for_inspect(k) : values[k].inspect
-
-                  # fix for https://github.com/rails/rails/issues/40725
-                  # TODO only apply to Active Record 6.0
-                  if respond_to?(:inspection_filter, true) && v != "nil"
-                    v = inspection_filter.filter_param(k, v)
-                  end
                 else
                   next
                 end
@@ -137,15 +131,51 @@ module Lockbox
                 # essentially a no-op if already loaded
                 # an exception is thrown if decryption fails
                 self.class.lockbox_attributes.each do |_, lockbox_attribute|
-                  # don't try to decrypt if no decryption key given
-                  next if lockbox_attribute[:algorithm] == "hybrid" && lockbox_attribute[:decryption_key].nil?
-
                   # it is possible that the encrypted attribute is not loaded, eg.
                   # if the record was fetched partially (`User.select(:id).first`).
                   # accessing a not loaded attribute raises an `ActiveModel::MissingAttributeError`.
-                  send(lockbox_attribute[:attribute]) if has_attribute?(lockbox_attribute[:encrypted_attribute])
+                  if has_attribute?(lockbox_attribute[:encrypted_attribute])
+                    begin
+                      send(lockbox_attribute[:attribute])
+                    rescue ArgumentError => e
+                      raise e if e.message != "No decryption key set"
+                    end
+                  end
+                end
+
+                # remove attributes that do not have a ciphertext attribute
+                attributes = super
+                self.class.lockbox_attributes.each do |k, lockbox_attribute|
+                  if !attributes.include?(lockbox_attribute[:encrypted_attribute].to_s)
+                    attributes.delete(k.to_s)
+                    attributes.delete(lockbox_attribute[:attribute])
+                  end
+                end
+                attributes
+              end
+
+              # remove attribute names that do not have a ciphertext attribute
+              def attribute_names
+                # hash preserves key order
+                names_set = super.to_h { |v| [v, true] }
+                self.class.lockbox_attributes.each do |k, lockbox_attribute|
+                  if !names_set.include?(lockbox_attribute[:encrypted_attribute].to_s)
+                    names_set.delete(k.to_s)
+                    names_set.delete(lockbox_attribute[:attribute])
+                  end
+                end
+                names_set.keys
+              end
+
+              # check the ciphertext attribute for encrypted attributes
+              def has_attribute?(attr_name)
+                attr_name = attr_name.to_s
+                _, lockbox_attribute = self.class.lockbox_attributes.find { |_, la| la[:attribute] == attr_name }
+                if lockbox_attribute
+                  super(lockbox_attribute[:encrypted_attribute])
+                else
+                  super
                 end
-                super
               end
 
               # needed for in-place modifications
@@ -229,50 +259,69 @@ module Lockbox
                 result
               end
 
-              if ActiveRecord::VERSION::MAJOR >= 6
-                def self.insert_all(attributes, **options)
-                  super(lockbox_map_attributes(attributes), **options)
+              if ActiveRecord::VERSION::STRING.to_f >= 7.2
+                def self.insert(attributes, **options)
+                  super(lockbox_map_record_attributes(attributes), **options)
                 end
 
-                def self.insert_all!(attributes, **options)
-                  super(lockbox_map_attributes(attributes), **options)
+                def self.insert!(attributes, **options)
+                  super(lockbox_map_record_attributes(attributes), **options)
                 end
 
-                def self.upsert_all(attributes, **options)
-                  super(lockbox_map_attributes(attributes, check_readonly: true), **options)
+                def self.upsert(attributes, **options)
+                  super(lockbox_map_record_attributes(attributes, check_readonly: true), **options)
+                end
+              end
+
+              def self.insert_all(attributes, **options)
+                super(lockbox_map_attributes(attributes), **options)
+              end
+
+              def self.insert_all!(attributes, **options)
+                super(lockbox_map_attributes(attributes), **options)
+              end
+
+              def self.upsert_all(attributes, **options)
+                super(lockbox_map_attributes(attributes, check_readonly: true), **options)
+              end
+
+              # private
+              # does not try to handle :returning option for simplicity
+              def self.lockbox_map_attributes(records, check_readonly: false)
+                return records unless records.is_a?(Array)
+
+                records.map do |attributes|
+                  lockbox_map_record_attributes(attributes, check_readonly: false)
+                end
+              end
+
+              # private
+              def self.lockbox_map_record_attributes(attributes, check_readonly: false)
+                return attributes unless attributes.is_a?(Hash)
+
+                # transform keys like Active Record
+                attributes = attributes.transform_keys do |key|
+                  n = key.to_s
+                  attribute_aliases[n] || n
                 end
 
-                # private
-                # does not try to handle :returning option for simplicity
-                def self.lockbox_map_attributes(records, check_readonly: false)
-                  return records unless records.is_a?(Array)
-
-                  records.map do |attributes|
-                    # transform keys like Active Record
-                    attributes = attributes.transform_keys do |key|
-                      n = key.to_s
-                      attribute_aliases[n] || n
-                    end
-
-                    lockbox_attributes = self.lockbox_attributes.slice(*attributes.keys.map(&:to_sym))
-                    lockbox_attributes.each do |key, lockbox_attribute|
-                      attribute = key.to_s
-                      # check read only
-                      # users should mark both plaintext and ciphertext columns
-                      if check_readonly && readonly_attributes.include?(attribute) && !readonly_attributes.include?(lockbox_attribute[:encrypted_attribute].to_s)
-                        warn "[lockbox] WARNING: Mark attribute as readonly: #{lockbox_attribute[:encrypted_attribute]}"
-                      end
-
-                      message = attributes[attribute]
-                      attributes.delete(attribute) unless lockbox_attribute[:migrating]
-                      encrypted_attribute = lockbox_attribute[:encrypted_attribute]
-                      ciphertext = send("generate_#{encrypted_attribute}", message)
-                      attributes[encrypted_attribute] = ciphertext
-                    end
-
-                    attributes
+                lockbox_attributes = self.lockbox_attributes.slice(*attributes.keys.map(&:to_sym))
+                lockbox_attributes.each do |key, lockbox_attribute|
+                  attribute = key.to_s
+                  # check read only
+                  # users should mark both plaintext and ciphertext columns
+                  if check_readonly && readonly_attributes.include?(attribute) && !readonly_attributes.include?(lockbox_attribute[:encrypted_attribute].to_s)
+                    warn "[lockbox] WARNING: Mark attribute as readonly: #{lockbox_attribute[:encrypted_attribute]}"
                   end
+
+                  message = attributes[attribute]
+                  attributes.delete(attribute) unless lockbox_attribute[:migrating]
+                  encrypted_attribute = lockbox_attribute[:encrypted_attribute]
+                  ciphertext = send("generate_#{encrypted_attribute}", message)
+                  attributes[encrypted_attribute] = ciphertext
                 end
+
+                attributes
               end
             else
               def reload
@@ -289,17 +338,22 @@ module Lockbox
           @lockbox_attributes[original_name] = options
 
           if activerecord
+            # warn on store attributes
+            if stored_attributes.any? { |k, v| v.include?(name) }
+              warn "[lockbox] WARNING: encrypting store accessors is not supported. Encrypt the column instead."
+            end
+
             # warn on default attributes
-            if attributes_to_define_after_schema_loads.key?(name.to_s)
+            if ActiveRecord::VERSION::STRING.to_f >= 7.2
+              # TODO improve
+              if pending_attribute_modifications.any? { |v| v.is_a?(ActiveModel::AttributeRegistration::ClassMethods::PendingDefault) && v.name == name.to_s }
+                warn "[lockbox] WARNING: attributes with `:default` option are not supported. Use `after_initialize` instead."
+              end
+            elsif attributes_to_define_after_schema_loads.key?(name.to_s)
               opt = attributes_to_define_after_schema_loads[name.to_s][1]
 
-              has_default =
-                if ActiveRecord::VERSION::MAJOR >= 7
-                  # not ideal, since NO_DEFAULT_PROVIDED is private
-                  opt != ActiveRecord::Attributes::ClassMethods.const_get(:NO_DEFAULT_PROVIDED)
-                else
-                  opt.is_a?(Hash) && opt.key?(:default)
-                end
+              # not ideal, since NO_DEFAULT_PROVIDED is private
+              has_default = opt != ActiveRecord::Attributes::ClassMethods.const_get(:NO_DEFAULT_PROVIDED)
 
               if has_default
                 warn "[lockbox] WARNING: attributes with `:default` option are not supported. Use `after_initialize` instead."
@@ -323,14 +377,33 @@ module Lockbox
 
               attribute name, attribute_type
 
-              if ActiveRecord::VERSION::STRING.to_f >= 7.1
-                serialize name, coder: JSON if options[:type] == :json
-                serialize name, type: Hash if options[:type] == :hash
-                serialize name, type: Array if options[:type] == :array
-              else
-                serialize name, JSON if options[:type] == :json
-                serialize name, Hash if options[:type] == :hash
-                serialize name, Array if options[:type] == :array
+              case options[:type]
+              when :json
+                serialize name, coder: JSON
+              when :hash
+                serialize name, type: Hash, coder: default_column_serializer || YAML
+              when :array
+                serialize name, type: Array, coder: default_column_serializer || YAML
+              end
+            elsif ActiveRecord::VERSION::STRING.to_f >= 7.2
+              decorate_attributes([name]) do |attr_name, cast_type|
+                if cast_type.instance_of?(ActiveRecord::Type::Value)
+                  original_type = pending_attribute_modifications.find { |v| v.is_a?(ActiveModel::AttributeRegistration::ClassMethods::PendingType) && v.name == original_name.to_s && !v.type.nil? }&.type
+                  if original_type
+                    original_type
+                  elsif options[:migrating]
+                    cast_type
+                  else
+                    ActiveRecord::Type::String.new
+                  end
+                elsif cast_type.is_a?(ActiveRecord::Type::Serialized) && cast_type.subtype.instance_of?(ActiveModel::Type::Value)
+                  # hack to set string type after serialize
+                  # otherwise, type gets set to ActiveModel::Type::Value
+                  # which always returns false for changed_in_place?
+                  ActiveRecord::Type::Serialized.new(ActiveRecord::Type::String.new, cast_type.coder)
+                else
+                  cast_type
+                end
               end
             elsif !attributes_to_define_after_schema_loads.key?(name.to_s)
               # when migrating it's best to specify the type directly
@@ -344,22 +417,14 @@ module Lockbox
               else
                 attribute name, :string
               end
-            else
-              # hack for Active Record 6.1
-              # to set string type after serialize
+            elsif attributes_to_define_after_schema_loads[name.to_s].first.is_a?(Proc)
+              # hack for Active Record 6.1+ to set string type after serialize
               # otherwise, type gets set to ActiveModel::Type::Value
               # which always returns false for changed_in_place?
               # earlier versions of Active Record take the previous code path
-              if ActiveRecord::VERSION::STRING.to_f >= 7.0 && attributes_to_define_after_schema_loads[name.to_s].first.is_a?(Proc)
-                attribute_type = attributes_to_define_after_schema_loads[name.to_s].first.call(nil)
-                if attribute_type.is_a?(ActiveRecord::Type::Serialized) && attribute_type.subtype.nil?
-                  attribute name, ActiveRecord::Type::Serialized.new(ActiveRecord::Type::String.new, attribute_type.coder)
-                end
-              elsif ActiveRecord::VERSION::STRING.to_f >= 6.1 && attributes_to_define_after_schema_loads[name.to_s].first.is_a?(Proc)
-                attribute_type = attributes_to_define_after_schema_loads[name.to_s].first.call
-                if attribute_type.is_a?(ActiveRecord::Type::Serialized) && attribute_type.subtype.nil?
-                  attribute name, ActiveRecord::Type::Serialized.new(ActiveRecord::Type::String.new, attribute_type.coder)
-                end
+              attribute_type = attributes_to_define_after_schema_loads[name.to_s].first.call(nil)
+              if attribute_type.is_a?(ActiveRecord::Type::Serialized) && attribute_type.subtype.nil?
+                attribute name, ActiveRecord::Type::Serialized.new(ActiveRecord::Type::String.new, attribute_type.coder)
               end
             end
 
@@ -432,12 +497,12 @@ module Lockbox
             # decrypt first for dirty tracking
             # don't raise error if can't decrypt previous
             # don't try to decrypt if no decryption key given
-            unless options[:algorithm] == "hybrid" && options[:decryption_key].nil?
-              begin
-                send(name)
-              rescue Lockbox::DecryptionError
-                warn "[lockbox] Decrypting previous value failed"
-              end
+            begin
+              send(name)
+            rescue Lockbox::DecryptionError
+              warn "[lockbox] Decrypting previous value failed"
+            rescue ArgumentError => e
+              raise e if e.message != "No decryption key set"
             end
 
             send("lockbox_direct_#{name}=", message)
@@ -499,6 +564,9 @@ module Lockbox
                     clear_attribute_change(name)
                   end
                 end
+
+                # ensure same object is returned as next call
+                message = super()
               else
                 instance_variable_set("@#{name}", message)
               end
@@ -515,7 +583,6 @@ module Lockbox
               case options[:type]
               when :boolean
                 message = ActiveRecord::Type::Boolean.new.serialize(message)
-                message = nil if message == "" # for Active Record < 5.2
                 message = message ? "t" : "f" unless message.nil?
               when :date
                 message = ActiveRecord::Type::Date.new.serialize(message)
@@ -523,7 +590,6 @@ module Lockbox
                 message = message.strftime("%Y-%m-%d") unless message.nil?
               when :datetime
                 message = ActiveRecord::Type::DateTime.new.serialize(message)
-                message = nil unless message.respond_to?(:iso8601) # for Active Record < 5.2
                 message = message.iso8601(9) unless message.nil?
               when :time
                 message = ActiveRecord::Type::Time.new.serialize(message)
@@ -540,14 +606,7 @@ module Lockbox
                 # double precision, big endian
                 message = [message].pack("G") unless message.nil?
               when :decimal
-                message =
-                  if ActiveRecord::VERSION::MAJOR >= 6
-                    ActiveRecord::Type::Decimal.new.serialize(message)
-                  else
-                    # issue with serialize in Active Record < 6
-                    # https://github.com/rails/rails/commit/a741208f80dd33420a56486bd9ed2b0b9862234a
-                    ActiveRecord::Type::Decimal.new.cast(message)
-                  end
+                message = ActiveRecord::Type::Decimal.new.serialize(message)
                 # Postgres stores 4 decimal digits in 2 bytes
                 # plus 3 to 8 bytes of overhead
                 # but use string for simplicity
@@ -615,6 +674,10 @@ module Lockbox
               else
                 # use original name for serialized attributes if no type specified
                 type = (try(:attribute_types) || {})[(options[:type] ? name : original_name).to_s]
+                # for Action Text
+                if activerecord && type.is_a?(ActiveRecord::Type::Serialized) && defined?(ActionText::Content) && type.coder == ActionText::Content
+                  message.force_encoding(Encoding::UTF_8)
+                end
                 message = type.deserialize(message) if type
                 message.force_encoding(Encoding::UTF_8) if !type || type.is_a?(ActiveModel::Type::String)
               end
@@ -646,11 +709,6 @@ module Lockbox
       end
     end
 
-    def lockbox_encrypts(*attributes, **options)
-      ActiveSupport::Deprecation.warn("`#{__callee__}` is deprecated in favor of `has_encrypted`")
-      has_encrypted(*attributes, **options)
-    end
-
     module Attached
       def encrypts_attached(*attributes, **options)
         attributes.each do |name|
